@@ -1,5 +1,5 @@
 from collections import deque
-from typing import Callable
+from typing import Callable, List
 
 from ezcode.heap import PriorityMap
 
@@ -28,6 +28,9 @@ class Graph:
     def __len__(self) -> int:
         return len(self.nodes)
 
+    def build_graph(self, edges: List[list], weights: list = None):
+        pass
+
     def _cell(self, item=None) -> str:
         if not self.is_weighted and item == 1:
             item = self.mark
